package de.ozgcloud.nachrichten.postfach.osiv2;

import java.util.stream.Stream;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachRemoteService;
import de.ozgcloud.nachrichten.postfach.osiv2.transfer.PostfachApiFacadeService;
import lombok.extern.log4j.Log4j2;

@Service
@ConditionalOnProperty("ozgcloud.osiv2-postfach.enabled")
@Log4j2
public record OsiPostfachRemoteService(
		PostfachApiFacadeService postfachApiFacadeService
) implements PostfachRemoteService {
	public static final String POSTFACH_TYPE_OSI = "OSI";

	@Override
	public void sendMessage(PostfachNachricht nachricht) {
		try {
			postfachApiFacadeService.sendMessage(nachricht);
		} catch (RuntimeException e) {
			throw new OsiPostfachException("Failed to send message", e);
		}
	}

	@Override
	public Stream<PostfachNachricht> getAllMessages() {
		// TODO handle exceptions?
		return postfachApiFacadeService.receiveMessages();
	}

	@Override
	public void deleteMessage(String messageId) {
		// TODO
	}

	@Override
	public String getPostfachType() {
		return POSTFACH_TYPE_OSI;
	}

	@Override
	public boolean isReplyAllowed() {
		return true;
	}
}
