package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.util.stream.Stream;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.api.MessageExchangeApi;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@ConditionalOnProperty("ozgcloud.osiv2-postfach.enabled")
public record PostfachApiFacadeService(
		MessageExchangeApi messageExchangeApi,
		RequestMapper requestMapper,
		ResponseMapper responseMapper
) {
	public void sendMessage(PostfachNachricht nachricht) {
		messageExchangeApi.sendMessage(
				requestMapper.mapMailboxId(nachricht),
				requestMapper.mapOutSendMessageRequestV2(nachricht)
		).block();
	}

	public Stream<PostfachNachricht> receiveMessages() {
		// TODO fetch message by guid and map to PostfachNachricht
		var response = messageExchangeApi.receiveMessages(100, 0)
				.block();
		LOG.info("receive-message-guids: {}", String.join(", ",
				response.getMessages().stream().map(message -> message.getGuid().toString()).toList()));
		return Stream.empty();
	}

}
