package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachAddressIdentifier;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeFiles;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.OutSendMessageRequestV2;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1References;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR)
public interface RequestMapper {

	@Mapping(target = "sequencenumber", source = "vorgangId")
	@Mapping(target = "body", source = "mailBody")
	@Mapping(target = "displayName", source = "createdBy")
	@Mapping(target = "originSender", ignore = true)
	@Mapping(target = "replyAction", source = "replyOption")
	@Mapping(target = "eidasLevel", constant = "LOW")
	@Mapping(target = "isObligatory", expression = "java( false )")
	@Mapping(target = "isHtml", expression = "java( false )")
	@Mapping(target = "files", expression = "java( mapMessageExchangeFiles() )")
	@Mapping(target = "references", expression = "java( mapReferences() )")
	OutSendMessageRequestV2 mapOutSendMessageRequestV2(PostfachNachricht nachricht);

	default List<MessageExchangeFiles> mapMessageExchangeFiles() {
		return Collections.emptyList();
	}

	default List<V1References> mapReferences() {
		return Collections.emptyList();
	}

	default V1ReplyBehavior mapReplyAction(PostfachNachricht.ReplyOption replyOption) {
		return switch (replyOption) {
			case FORBIDDEN -> V1ReplyBehavior.REPLYFORBIDDEN;
			case POSSIBLE -> V1ReplyBehavior.REPLYPOSSIBLE;
			case MANDATORY -> V1ReplyBehavior.REPLYMANDATORY;
		};
	}

	default String mapMailboxId(PostfachNachricht nachricht) {
		return Optional.ofNullable(nachricht.getPostfachAddress())
				.map(PostfachAddress::getIdentifier)
				.filter(PostfachAddressIdentifier::isStringBasedIdentifier)
				.map(Object::toString)
				.orElseThrow(() -> new IllegalArgumentException(
						"Missing MailboxId! Expect MailboxId to be a string-based PostfachAddress of PostfachNachricht."));
	}

}
