package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.api.MessageExchangeApi;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeSendMessageResponse;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.OutSendMessageRequestV2;
import reactor.core.publisher.Mono;

class PostfachApiFacadeServiceTest {

	@InjectMocks
	PostfachApiFacadeService postfachApiFacadeService;

	@Mock
	MessageExchangeApi messageExchangeApi;

	@Mock
	RequestMapper requestMapper;

	@Mock
	ResponseMapper responseMapper;

	@DisplayName("send message")
	@Nested
	class TestSendMessage {

		@Mock
		OutSendMessageRequestV2 outSendMessageRequestV2;

		@Mock
		MessageExchangeSendMessageResponse messageExchangeSendMessageResponse;

		private final PostfachNachricht nachricht = PostfachNachrichtTestFactory.create();

		@BeforeEach
		void mock() {
			var responseMono = Mono.just(messageExchangeSendMessageResponse);
			when(requestMapper.mapMailboxId(nachricht)).thenReturn(MAILBOX_ID);
			when(requestMapper.mapOutSendMessageRequestV2(nachricht)).thenReturn(outSendMessageRequestV2);
			when(messageExchangeApi.sendMessage(any(), any())).thenReturn(responseMono);
		}

		@DisplayName("should call sendMessage")
		@Test
		void shouldCallSendMessage() {
			postfachApiFacadeService.sendMessage(nachricht);

			verify(messageExchangeApi).sendMessage(MAILBOX_ID, outSendMessageRequestV2);

		}
	}

}